@@ -29,6 +29,7 @@ enum {
 	ARG_WIDTH,
 	ARG_MINIMUM_WIDTH,
 	ARG_FROZEN,
+	ARG_TABLE_ALTERNATING_ROW_COLORS,
 	ARG_TABLE_HORIZONTAL_DRAW_GRID,
 	ARG_TABLE_VERTICAL_DRAW_GRID,
 	ARG_TABLE_DRAW_FOCUS,
@@ -46,8 +47,35 @@ etgl_destroy (GtkObject *object)
 	ETableGroupLeaf *etgl = E_TABLE_GROUP_LEAF(object);
 	if (etgl->ets)
 		gtk_object_unref (GTK_OBJECT(etgl->ets));
-	if (etgl->item)
+	if (etgl->item) {
+		if (etgl->etgl_cursor_change_id != 0)
+			gtk_signal_disconnect (GTK_OBJECT (etgl->item),
+					       etgl->etgl_cursor_change_id);
+		if (etgl->etgl_cursor_activated_id != 0)
+			gtk_signal_disconnect (GTK_OBJECT (etgl->item),
+					       etgl->etgl_cursor_activated_id);
+		if (etgl->etgl_double_click_id != 0)
+			gtk_signal_disconnect (GTK_OBJECT (etgl->item),
+					       etgl->etgl_double_click_id);
+		if (etgl->etgl_right_click_id != 0)
+			gtk_signal_disconnect (GTK_OBJECT (etgl->item),
+					       etgl->etgl_right_click_id);
+		if (etgl->etgl_click_id != 0)
+			gtk_signal_disconnect (GTK_OBJECT (etgl->item),
+					       etgl->etgl_click_id);
+		if (etgl->etgl_key_press_id != 0)   
+			gtk_signal_disconnect (GTK_OBJECT (etgl->item),
+					       etgl->etgl_key_press_id);
 		gtk_object_destroy (GTK_OBJECT(etgl->item));
+
+		etgl->etgl_cursor_change_id = 0;
+		etgl->etgl_cursor_activated_id = 0;
+		etgl->etgl_double_click_id = 0;
+		etgl->etgl_right_click_id = 0;
+		etgl->etgl_click_id = 0;
+		etgl->etgl_key_press_id = 0;
+		etgl->item = NULL;
+	}
 	if (etgl->selection_model)
 		gtk_object_unref (GTK_OBJECT(etgl->selection_model));
 	if (GTK_OBJECT_CLASS (etgl_parent_class)->destroy)
@@ -120,14 +148,13 @@ etgl_cursor_activated (GtkObject *object, gint row, ETableGroupLeaf *etgl)
 static void
 etgl_double_click (GtkObject *object, gint row, gint col, GdkEvent *event, ETableGroupLeaf *etgl)
 {
-	if (row < E_TABLE_SUBSET(etgl->ets)->n_map)
-		e_table_group_double_click (E_TABLE_GROUP(etgl), E_TABLE_SUBSET(etgl->ets)->map_table[row], col, event);
+	e_table_group_double_click (E_TABLE_GROUP(etgl), row, col, event);
 }
 
 static gint
 etgl_key_press (GtkObject *object, gint row, gint col, GdkEvent *event, ETableGroupLeaf *etgl)
 {
-	if (row < E_TABLE_SUBSET(etgl->ets)->n_map)
+	if (row < E_TABLE_SUBSET(etgl->ets)->n_map && row >= 0)
 		return e_table_group_key_press (E_TABLE_GROUP(etgl), E_TABLE_SUBSET(etgl->ets)->map_table[row], col, event);
 	else
 		return 0;
@@ -176,6 +203,7 @@ etgl_realize (GnomeCanvasItem *item)
 							 e_table_item_get_type (),
 							 "ETableHeader", E_TABLE_GROUP(etgl)->header,
 							 "ETableModel", etgl->ets,
+							 "alternating_row_colors", etgl->alternating_row_colors,
 							 "horizontal_draw_grid", etgl->horizontal_draw_grid,
 							 "vertical_draw_grid", etgl->vertical_draw_grid,
 							 "drawfocus", etgl->draw_focus,
@@ -185,18 +213,12 @@ etgl_realize (GnomeCanvasItem *item)
 							 "selection_model", etgl->selection_model,
 							 NULL));
 	
-	gtk_signal_connect (GTK_OBJECT(etgl->item), "cursor_change",
-			    GTK_SIGNAL_FUNC(etgl_cursor_change), etgl);
-	gtk_signal_connect (GTK_OBJECT(etgl->item), "cursor_activated",
-			    GTK_SIGNAL_FUNC(etgl_cursor_activated), etgl);
-	gtk_signal_connect (GTK_OBJECT(etgl->item), "double_click",
-			    GTK_SIGNAL_FUNC(etgl_double_click), etgl);
-	gtk_signal_connect (GTK_OBJECT(etgl->item), "right_click",
-			    GTK_SIGNAL_FUNC(etgl_right_click), etgl);
-	gtk_signal_connect (GTK_OBJECT(etgl->item), "click",
-			    GTK_SIGNAL_FUNC(etgl_click), etgl);
-	gtk_signal_connect (GTK_OBJECT(etgl->item), "key_press",
-			    GTK_SIGNAL_FUNC(etgl_key_press), etgl);
+	etgl->etgl_cursor_change_id    = gtk_signal_connect (GTK_OBJECT(etgl->item), "cursor_change", GTK_SIGNAL_FUNC(etgl_cursor_change), etgl);
+	etgl->etgl_cursor_activated_id = gtk_signal_connect (GTK_OBJECT(etgl->item), "cursor_activated", GTK_SIGNAL_FUNC(etgl_cursor_activated), etgl);
+	etgl->etgl_double_click_id     = gtk_signal_connect (GTK_OBJECT(etgl->item), "double_click", GTK_SIGNAL_FUNC(etgl_double_click), etgl);
+	etgl->etgl_right_click_id      = gtk_signal_connect (GTK_OBJECT(etgl->item), "right_click", GTK_SIGNAL_FUNC(etgl_right_click), etgl);
+	etgl->etgl_click_id            = gtk_signal_connect (GTK_OBJECT(etgl->item), "click", GTK_SIGNAL_FUNC(etgl_click), etgl);
+	etgl->etgl_key_press_id        = gtk_signal_connect (GTK_OBJECT(etgl->item), "key_press", GTK_SIGNAL_FUNC(etgl_key_press), etgl);
 	e_canvas_item_request_reflow(item);
 }
 
@@ -294,6 +316,13 @@ etgl_compute_location (ETableGroup *etg, int *x, int *y, int *row, int *col)
 	e_table_item_compute_location (etgl->item, x, y, row, col);
 }
 
+static void
+etgl_get_cell_geometry (ETableGroup *etg, int *row, int *col, int *x, int *y, int *width, int *height)
+{
+	ETableGroupLeaf *etgl = E_TABLE_GROUP_LEAF (etg);
+	e_table_item_get_cell_geometry (etgl->item, row, col, x, y, width, height);
+}
+
 static void
 etgl_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 {
@@ -336,6 +365,16 @@ etgl_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 					       "selection_model", etgl->selection_model,
 					       NULL);
 		}
+		break;
+
+	case ARG_TABLE_ALTERNATING_ROW_COLORS:
+		etgl->alternating_row_colors = GTK_VALUE_BOOL (*arg);
+		if (etgl->item) {
+			gnome_canvas_item_set (GNOME_CANVAS_ITEM(etgl->item),
+					       "alternating_row_colors", GTK_VALUE_BOOL (*arg),
+					       NULL);
+		}
+		break;
 
 	case ARG_TABLE_HORIZONTAL_DRAW_GRID:
 		etgl->horizontal_draw_grid = GTK_VALUE_BOOL (*arg);
@@ -427,7 +466,10 @@ etgl_class_init (GtkObjectClass *object_class)
 	e_group_class->get_focus_column = etgl_get_focus_column;
 	e_group_class->get_printable = etgl_get_printable;
 	e_group_class->compute_location = etgl_compute_location;
+	e_group_class->get_cell_geometry = etgl_get_cell_geometry;
 
+	gtk_object_add_arg_type ("ETableGroupLeaf::alternating_row_colors", GTK_TYPE_BOOL,
+				 GTK_ARG_WRITABLE, ARG_TABLE_ALTERNATING_ROW_COLORS);
 	gtk_object_add_arg_type ("ETableGroupLeaf::horizontal_draw_grid", GTK_TYPE_BOOL,
 				 GTK_ARG_WRITABLE, ARG_TABLE_HORIZONTAL_DRAW_GRID);
 	gtk_object_add_arg_type ("ETableGroupLeaf::vertical_draw_grid", GTK_TYPE_BOOL,
@@ -462,7 +504,15 @@ etgl_init (GtkObject *object)
 
 	etgl->ets = NULL;
 	etgl->item = NULL;
-	
+
+	etgl->etgl_cursor_change_id = 0;
+	etgl->etgl_cursor_activated_id = 0;
+	etgl->etgl_double_click_id = 0;
+	etgl->etgl_right_click_id = 0;
+	etgl->etgl_click_id = 0;
+	etgl->etgl_key_press_id = 0;
+
+	etgl->alternating_row_colors = 1;
 	etgl->horizontal_draw_grid = 1;
 	etgl->vertical_draw_grid = 1;
 	etgl->draw_focus = 1;
