@@ -217,6 +217,7 @@ ets_proxy_model_rows_inserted (ETableSubset *etss, ETableModel *source, int row,
 
 		e_table_model_row_inserted (etm, i);
 		d(g_print("inserted row %d", row));
+		row++;
 	}
 	d(e_table_subset_print_debugging(etss));
 }
@@ -229,11 +230,11 @@ ets_proxy_model_rows_deleted (ETableSubset *etss, ETableModel *source, int row,
 	gboolean shift;
 	int j;
 
-	shift = row == etss->n_map - 1;
+	shift = row == etss->n_map - count;
 	
 	for (j = 0; j < count; j++) {
 		for (i = 0; i < etss->n_map; i++){
-			if (etss->map_table[i] == row) {
+			if (etss->map_table[i] == row + j) {
 				e_table_model_pre_change (etm);
 				memmove (etss->map_table + i, etss->map_table + i + 1, (etss->n_map - i - 1) * sizeof(int));
 				etss->n_map --;
